Module cache

Module cache 

This module contains everything related to games that cache intermediate search results to improve performance.

Structs

Cached
This struct keeps track of a cache for a Game, which allows it to store intermediate search results. It can be created with the WithCache::with_cache method.

Traits

WithCache
This trait provides the WithCache::with_cache method, which is implemented for all types that implement Game and satisfy the trait bounds necessary to store intermediate results in cache: Clone, Eq, and Hash.