pub enum Hint {
Execute,
Skip,
Alias(u8),
KnownBool(bool),
}Expand description
Hints guide the execution of individual instructions in the machine.
Variants
Execute
Instruction executes normally.
Skip
Instruction is not needed for the outputs being computed. Examples include dead code or an untaken branch.
Alias(u8)
Skip execution and copy the value from the chosen input position.
For example, if (true) x else y becomes an alias to x.
KnownBool(bool)
Skip execution because the result is already known exactly.
For example, if (x > 0) where x = [5, 10] is always true.