pub enum RivalError {
InvalidInput,
Unsamplable,
}Expand description
Errors that can occur during Machine::apply.
Note that Machine::apply will only return a result if it can prove
that it has correctly-rounded the output, and it will only return
RivalError::InvalidInput if it can prove that at least one of the
output expressions in the machine throws on the given input.
Variants
InvalidInput
The input point is invalid for at least one compiled expression.
For example, taking the square root of a negative number, or dividing by zero.
Unsamplable
Rival was unable to correctly round the output within the configured precision and iteration limits.