ErrorFlags

Struct ErrorFlags 

pub struct ErrorFlags { /* private fields */ }
Flags indicating whether invalid inputs were discarded during computation.

When an interval contains invalid inputs to some function (e.g., negative inputs to sqrt), these flags record what happened:

  • partial: at least one invalid input was discarded, but some valid inputs remain.
  • total: all inputs were invalid.

Error flags are “sticky”: further computations on an interval will maintain already-set error flags.

Implementations

Source

impl ErrorFlags

pub fn new(partial: bool, total: bool) -> Self

pub fn none() -> Self

pub fn error() -> Self

pub fn partial(&self) -> bool

pub fn total(&self) -> bool