pub struct ErrorFlags { /* private fields */ }Expand description
Flags indicating whether invalid inputs were discarded during computation.
When an interval contains invalid inputs to some function (e.g.,
negative inputs to sqrt), these flags record what happened:
partial: at least one invalid input was discarded, but some valid inputs remain.total: all inputs were invalid.
Error flags are “sticky”: further computations on an interval will maintain already-set error flags.
Implementations