pub struct Execution {
pub name: &'static str,
pub number: i32,
pub precision: u32,
pub time_ms: f64,
pub iteration: usize,
}Expand description
A single recorded execution of a Rival interval operator.
Each execution corresponds to a single interval operator being executed.
The name names the operator, except the special
name "adjust", in which case it refers to an internal precision-tuning
pass.
The number gives its position in the compiled
instruction sequence; this allows disambiguating if an expression contains,
say, multiple addition operations.
The precision is the working precision in bits
that the operator was executed at,
the time_ms is the time, in milliseconds, that
the execution took,
and the iteration records which sampling
iteration triggered the execution.
Note that, because Rival executes the register machine multiple times, the same operator (with the same name and number) can appear multiple times for a single point. On the other hand, in some iterations Rival might skip some operators, if the precision is unchanged from previous iterations, so not every operator may show up in the executions list the same number of times.
Fields
name: &'static strThe name of the operator, or "adjust" for precision-tuning passes.
number: i32Position in the compiled instruction sequence (-1 for adjust passes).
precision: u32Working precision in bits for this execution.
time_ms: f64Wall-clock time in milliseconds.
iteration: usizeThe sampling iteration that triggered this execution.